"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ScriptManager = void 0;
const precacheTargetList = [
    "audio", "document", "embed", "fetch", "font", "image", " object",
    "script", "style", "track", "video", "worker",
    // umd will need to convert to script to cache
    "umd",
];
/**
 * Helper class to manage loading of script elements. Only loads a given script once.
 */
class ScriptManager {
    // Check whether the script is loaded inside a worker.
    static get isBrowser() {
        if (typeof window === "undefined") {
            return false;
        }
        return window.document !== undefined;
    }
    static async protectEntrypoint(library, callback) {
        if (!this.isBrowser) {
            return callback();
        }
        // if we are in a browser we need to ensure multiple
        // scripts aren't loaded with the same entry point
        // otherwise they could overwrite each other
        while (this.loadingEntrypoints.has(library)) {
            await this.loadingEntrypoints.get(library);
        }
        const returnP = callback().finally(() => this.loadingEntrypoints.delete(library));
        this.loadingEntrypoints.set(library, returnP);
        return returnP;
    }
    static async internalLoadScript(scriptUrl, library) {
        if (!this.loadCache.has(scriptUrl)) {
            while (this.loadingEntrypoints.has(library)) {
                await this.loadingEntrypoints.get(library);
            }
            const scriptP = new Promise((resolve, reject) => {
                if (this.isBrowser) {
                    const script = document.createElement("script");
                    script.src = scriptUrl;
                    // Dynamically added scripts are async by default. By setting async to false, we are enabling the
                    // scripts to be downloaded in parallel, but executed in order. This ensures that a script is
                    // executed after all of its dependencies have been loaded and executed.
                    script.async = false;
                    script.onload = () => resolve(window[library]);
                    script.onerror = () => reject(new Error(`Failed to download the script at url: ${scriptUrl}`));
                    document.head.appendChild(script);
                }
                else {
                    Promise.resolve().then(() => __importStar(require(/* webpackMode: "eager", webpackIgnore: true */ scriptUrl))).then((value) => {
                        resolve(value);
                    }, () => {
                        reject(new Error(`Failed to download the script at url: ${scriptUrl}`));
                    });
                }
            });
            this.loadCache.set(scriptUrl, scriptP);
        }
        return this.loadCache.get(scriptUrl);
    }
    async preCacheFiles(browser) {
        var _a;
        const preCachePs = [];
        const docfrag = document.createDocumentFragment();
        for (const targetName of Object.keys(browser).filter((t) => precacheTargetList.includes(t))) {
            const target = browser[targetName];
            if (target !== undefined) {
                for (const url of target.files) {
                    const cacheLink = document.createElement("link");
                    cacheLink.href = url;
                    cacheLink.as = targetName === "umd" ? "script" : targetName;
                    cacheLink.crossOrigin = "anonymous";
                    if (((_a = cacheLink.relList) === null || _a === void 0 ? void 0 : _a.supports("preload")) === true) {
                        cacheLink.rel = "preload";
                    }
                    else {
                        cacheLink.rel = "prefetch";
                    }
                    const loadP = new Promise((resolve, reject) => {
                        cacheLink.onload = () => resolve();
                        // eslint-disable-next-line prefer-promise-reject-errors
                        cacheLink.onerror = (...args) => reject(Object.assign({}, args));
                    });
                    docfrag.appendChild(docfrag);
                    preCachePs.push(loadP);
                }
            }
        }
        document.appendChild(docfrag);
        await Promise.all(preCachePs);
    }
    async loadScript(scriptUrl, entryPoint) {
        return ScriptManager.protectEntrypoint(entryPoint, async () => ScriptManager.internalLoadScript(scriptUrl, entryPoint));
    }
    async loadLibrary(libraryDetails) {
        return ScriptManager.protectEntrypoint(libraryDetails.library, async () => {
            return Promise.all(libraryDetails.files.map(async (file) => ({
                file,
                entryPoint: await ScriptManager.internalLoadScript(file, libraryDetails.library),
            })));
        });
    }
}
exports.ScriptManager = ScriptManager;
ScriptManager.loadingEntrypoints = new Map();
ScriptManager.loadCache = new Map();
//# sourceMappingURL=scriptManager.js.map